import pandas as pd


def load_sensor(filepath):
    if filepath is None:
        return None
    result = pd.read_csv(filepath, parse_dates=[0], infer_datetime_format=True)
    result = result.rename(columns = {result.columns[0]: 'HEADER_TIME_STAMP'})
    return result


def load_annotation(filepath):
    if filepath is None:
        return None
    result = pd.read_csv(filepath, parse_dates=[0, 1, 2],
                         infer_datetime_format=True)
    return result

def load_offset_mapping(filepath):
    if filepath is None:
        return None
    result = pd.read_csv(filepath)
    return result

def load_orientation_corrections(filepath):
    if filepath is None:
        return None
    result = pd.read_csv(filepath)
    return result


from .. import dataset


def test_is_mhealth_filepath():
    correct_test_cases = [
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\2015\\11\\19\\16',
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\2015\\11\\19\\16\\',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/Derived/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/Derived/AllSensors/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16',
        '''D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16/
        ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-AccelerationCalibrated.
        2015-11-19-16-00-00-000-M0500.sensor.csv''',
    ]

    incorrect_test_cases = [
        'C:\\',
        'C:/',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/16',
        'D:/data/spades_lab/SPADES_7/MasterSynced/20/16',
        'D:/data/spades_lab/SPADES_7/Mastenced/2015/16/17/21/'
    ]
    for test_case in correct_test_cases:
        assert dataset.is_mhealth_filepath(test_case)

    for test_case in incorrect_test_cases:
        assert not dataset.is_mhealth_filepath(test_case)


def test_is_mhealth_flat_filepath():
    correct_test_cases = [
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\asdf.csv',
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\adfa.csv',
        'D:/data/spades_lab/SPADES_7/MasterSynced/sdf.csv',
        'D:/data/spades_lab/SPADES_7/Derived/dfew.csv',
        'D:/data/spades_lab/SPADES_7/Derived/AllSensors/dfsd.csv',
        '''D:/data/spades_lab/SPADES_7/MasterSynced/ActigraphGT9X\
        -AccelerationCalibrated-NA.TAS1E23150152-AccelerationCalibrated.\
        2015-11-19-16-00-00-000-M0500.sensor.csv''',
    ]

    incorrect_test_cases = [
        'C:\\',
        'C:/',
        'D:/data/spades_lab/SPADES_7/MasterSyn/',
        'D:/data/spades_lab/SPADES_7/MasterSynced/20/16',
        'D:/data/spades_lab/SPADES_7/Mastenced/2015/16/17/21/',
        '''D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16/\
        ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv'''
    ]
    for test_case in correct_test_cases:
        assert dataset.is_mhealth_flat_filepath(test_case)

    for test_case in incorrect_test_cases:
        assert not dataset.is_mhealth_flat_filepath(test_case)


def test_is_mhealth_filename():
    correct_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    incorrect_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.annotation.csv',
        'Actig?raphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
        'ActigraphGT9X-NA.TAS1E23150152-AccelerationCalibrated.\
        2015-11-19-16-00-00-000-M0500.sensor.csv',
        'ActigraphGT9X-AccelerationCalibrated-0,1,2.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
        'ActigraphGT9X-AccelerationCalibrated-NA.tas1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-0-000-M0500.sensor.csv',
        'SPADESInLab-sdfsdf.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv'
    ]

    for test_case in correct_test_cases:
        assert dataset.is_mhealth_filename(test_case)

    for test_case in incorrect_test_cases:
        assert not dataset.is_mhealth_filename(test_case)


def test_get_pid():
    correct_test_cases = [
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\2015\\11\\19\\16',
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\2015\\11\\19\\16\\',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/Derived/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/Derived/AllSensors/2015/11/19/16/',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16',
        '''D:/data/spades_lab/SPADES_7/MasterSynced/2015/11/19/16/
        ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-AccelerationCalibrated.
        2015-11-19-16-00-00-000-M0500.sensor.csv''',
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\asdf.csv',
        'D:\\data\\spades_lab\\SPADES_7\\MasterSynced\\adfa.csv',
        'D:/data/spades_lab/SPADES_7/MasterSynced/sdf.csv',
        'D:/data/spades_lab/SPADES_7/Derived/dfew.csv',
        'D:/data/spades_lab/SPADES_7/Derived/AllSensors/dfsd.csv',
        '''D:/data/spades_lab/SPADES_7/MasterSynced/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150152-AccelerationCalibrated.\
        2015-11-19-16-00-00-000-M0500.sensor.csv'''
    ]

    incorrect_test_cases = [
        'C:\\',
        'C:/',
        'D:/data/spades_lab/SPADES_7/MasterSynced/2015/16',
        'D:/data/spades_lab/SPADES_7/MasterSynced/20/16',
        'D:/data/spades_lab/SPADES_7/Mastenced/2015/16/17/21/'
    ]

    for test_case in correct_test_cases:
        assert dataset.get_pid(test_case) == 'SPADES_7'

    for test_case in incorrect_test_cases:
        assert dataset.get_pid(test_case) is None


def test_get_sensor_type():
    correct_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in correct_test_cases:
        print(test_case)
        assert dataset.get_sensor_type(
            test_case) == 'ActigraphGT9X' or \
            dataset.get_sensor_type(test_case) == 'SPADESInLab'


def test_get_data_type():
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_data_type(test_case) == 'AccelerationCalibrated'

    for test_case in annotation_test_cases:
        assert dataset.get_data_type(test_case) is None


def test_get_version_code():
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_version_code(test_case) == 'NA'

    for test_case in annotation_test_cases:
        assert dataset.get_version_code(test_case) is None


def test_get_sid():
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_sid(test_case) == 'TAS1E23150152'

    for test_case in annotation_test_cases:
        assert dataset.get_sid(test_case).lower() == 'diego'


def test_get_file_type():
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_file_type(test_case) == 'sensor'

    for test_case in annotation_test_cases:
        assert dataset.get_file_type(test_case) == 'annotation'


def test_get_file_timestamp():
    from datetime import datetime
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_file_timestamp(
            test_case).timestamp() == 1447966800.0

    for test_case in annotation_test_cases:
        assert dataset.get_file_timestamp(
            test_case).timestamp() == 1447966800.0


def test_get_timezone_name():
    from datetime import datetime
    sensor_test_cases = [
        'ActigraphGT9X-AccelerationCalibrated-NA.TAS1E23150152-\
        AccelerationCalibrated.2015-11-19-16-00-00-000-M0500.sensor.csv',
    ]

    annotation_test_cases = [
        'SPADESInLab.diego-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
        'SPADESInLab.DIEGO-SPADESInLab.2015-11-19-16-00-00-000-M0500.\
        annotation.csv',
    ]

    for test_case in sensor_test_cases:
        assert dataset.get_timezone_name(test_case) == 'UTC-05:00'

    for test_case in annotation_test_cases:
        assert dataset.get_timezone_name(test_case) == 'UTC-05:00'


def test_get_session_times():
    test_file_list = [
        "./SPADES_1/MasterSynced/2015/09/24/14/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150066-AccelerationCalibrated.\
        2015-09-24-14-23-00-000-M0400.sensor.csv",
        "./SPADES_1/MasterSynced/2015/09/24/16/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150175-AccelerationCalibrated.\
        2015-09-24-16-00-00-000-M0400.sensor.csv",
        "./SPADES_10/MasterSynced/2015/12/07/18/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150066-AccelerationCalibrated.\
        2015-12-07-18-00-00-000-M0500.sensor.csv",
        "./SPADES_10/MasterSynced/2015/12/07/19/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150075-AccelerationCalibrated.\
        2015-12-07-18-00-00-000-M0500.sensor.csv",
        "./SPADES_11/MasterSynced/2015/12/11/17/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150075-AccelerationCalibrated.\
        2015-12-11-17-00-00-000-M0500.sensor.csv",
        "./SPADES_11/MasterSynced/2015/12/11/18/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150084-AccelerationCalibrated.\
        2015-12-11-17-00-00-000-M0500.sensor.csv",
        "./SPADES_12/MasterSynced/2015/12/14/11/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150075-AccelerationCalibrated.\
        2015-12-14-11-15-00-000-M0500.sensor.csv",
        "./SPADES_12/MasterSynced/2015/12/15/11/ActigraphGT9X-\
        AccelerationCalibrated-NA.TAS1E23150084-AccelerationCalibrated.\
        2015-12-14-11-15-00-000-M0500.sensor.csv"
    ]
    start_time = dataset.get_session_start_time(
        test_file_list[0], test_file_list)
    end_time = dataset.get_session_end_time(test_file_list[0], test_file_list)
    assert start_time.strftime('%Y-%m-%d-%H-%M-%S') == '2015-09-24-14-23-00'
    assert end_time.strftime('%Y-%m-%d-%H-%M-%S') == '2015-09-24-16-00-00'



import chardet
import os
import pandas as pd


def detect_encoding(file_path):
    data = open(file_path, "rb").read()
    encoding = chardet.detect(data)
    if encoding['encoding'] == 'utf-8':
        return 'utf-8-sig'
    else:
        return encoding['encoding']


def is_file(input_data):
    return isinstance(input_data, str) and os.path.isfile(input_data)


def is_dataframe(input_data):
    return isinstance(input_data, pd.DataFrame.__class__)



import pandas as pd


def append_times(df, st, et):
    df.insert(0, 'START_TIME', st)
    df.insert(1, 'STOP_TIME', et)
    df = df.set_index(['START_TIME', 'STOP_TIME'])
    return df


def offset(df, offset_in_secs, start_time_col=0, stop_time_col=None):
    df_copy = df.copy(deep=True)
    if start_time_col is not None:
        start_time_col = df_copy.columns[start_time_col]
        df_copy[start_time_col] = df_copy[start_time_col] + \
            pd.Timedelta(offset_in_secs, unit='s')
    if stop_time_col is not None:
        stop_time_col = df_copy.columns[stop_time_col]
        df_copy[stop_time_col] = df_copy[stop_time_col] + \
            pd.Timedelta(offset_in_secs, unit='s')
    return df_copy


def segment(df, start_time=None, stop_time=None, start_time_col=0,
            stop_time_col=None):
    if stop_time_col is None:
        stop_time_col = start_time_col
    if start_time is None:
        start_time = df.iloc[0, start_time_col]
    if stop_time is None:
        stop_time = df.iloc[-1, stop_time_col]

    if start_time_col == stop_time_col:
        mask = (df.iloc[:, start_time_col] >= start_time) & (
            df.iloc[:, stop_time_col] < stop_time)
        return df[mask].copy(deep=True)
    else:
        mask = (df.iloc[:, start_time_col] < stop_time) & (
            df.iloc[:, stop_time_col] > start_time)
        subset_df = df[mask].copy(deep=True)
        subset_df[subset_df.iloc[:, start_time_col] <
                  start_time].iloc[:, start_time_col] = start_time
        subset_df[subset_df.iloc[:, stop_time_col] >
                  stop_time].iloc[:, stop_time_col] = stop_time
        return subset_df


def segment_sensor(df, start_time=None, stop_time=None):
    return segment(df, start_time=start_time, stop_time=stop_time)


def segment_annotation(df, start_time=None, stop_time=None):
    return segment(df, start_time=start_time, stop_time=stop_time,
                   start_time_col=1, stop_time_col=2)


def start_time(df, start_time_col=0):
    return df.iloc[0, start_time_col]


def end_time(df, stop_time_col=0):
    return df.iloc[-1, stop_time_col]


def append_edges(df, before_df=None, after_df=None, duration=120,
                 start_time_col=0, stop_time_col=0):
    lbound_time = df.iloc[0, start_time_col]
    rbound_time = df.iloc[-1, stop_time_col]

    if before_df is not None:
        ledge_df = segment(before_df,
                           start_time=lbound_time -
                           pd.Timedelta(duration, unit='s'),
                           stop_time=lbound_time,
                           start_time_col=start_time_col,
                           stop_time_col=stop_time_col)
    else:
        ledge_df = pd.DataFrame()

    if after_df is not None:
        redge_df = segment(after_df,
                           start_time=rbound_time,
                           stop_time=rbound_time +
                           pd.Timedelta(duration, unit='s'),
                           start_time_col=start_time_col,
                           stop_time_col=stop_time_col)
    else:
        redge_df = pd.DataFrame()

    return pd.concat((ledge_df, df, redge_df))



import re
import os
import datetime
import numpy as np
import pandas as pd
from glob import glob

CAMELCASE_PATTERN = r'(?:[A-Z][A-Za-z0-9]+)+'
VERSIONCODE_PATTERN = r'(?:NA|[0-9x]+)'
SID_PATTERN = r'[A-Z0-9]+'
ANNOTATOR_PATTERN = r'[A-Za-z0-9]+'
FILE_TIMESTAMP_PATTERN = r'[0-9]{4}(?:\-[0-9]{2}){5}-[0-9]{3}-(?:P|M)[0-9]{4}'
FILE_EXTENSION_PATTERN = r'''
(?:sensor|event|log|annotation|feature|class|prediction|model|classmap)\.csv
'''
MHEALTH_FLAT_FILEPATH_PATTERN = r'(\w+)[\/\\]{1}(?:(?:MasterSynced[\/\\]{1})|(?:Derived[\/\\]{1}(?:\w+[\/\\]{1})*))[0-9A-Za-z\-\.]+\.csv'
MHEALTH_FILEPATH_PATTERN = r'(\w+)[\/\\]{1}(?:(?:MasterSynced[\/\\]{1})|(?:Derived[\/\\]{1}(?:\w+[\/\\]{1})*))\d{4}[\/\\]{1}\d{2}[\/\\]{1}\d{2}[\/\\]{1}\d{2}'


def is_mhealth_filepath(filepath):
    filepath = os.path.abspath(filepath)
    matched = re.search(
        MHEALTH_FILEPATH_PATTERN,
        filepath)
    return matched is not None


def is_mhealth_flat_filepath(filepath):
    matched = re.search(
        MHEALTH_FLAT_FILEPATH_PATTERN,
        os.path.abspath(filepath)
    )
    return matched is not None


def get_mhealth_root(filepath):
    assert is_mhealth_filepath(filepath) or is_mhealth_flat_filepath(filepath)
    pid = get_pid(filepath)
    return filepath.split(pid)[0]


def get_pid_root(filepath):
    assert is_mhealth_filepath(filepath) or is_mhealth_flat_filepath(filepath)
    pid = get_pid(filepath)
    return os.path.join(filepath.split(pid)[0], pid)


def find_location_mapping(filepath):
    pid_root = get_pid_root(filepath)
    candicates = glob(os.path.join(
        pid_root, '**', 'location_mapping.csv'), recursive=True)
    if len(candicates) == 1:
        return candicates[0]
    else:
        mhealth_root = get_mhealth_root(filepath)
        result = os.path.join(
            mhealth_root, 'DerivedCrossParticipants', 'location_mapping.csv')
        if os.path.exists(result):
            return result
        else:
            return False


def find_offset_mapping(filepath):
    pid_root = get_pid_root(filepath)
    candicates = glob(os.path.join(
        pid_root, '**', 'offset_mapping.csv'), recursive=True)
    if len(candicates) == 1:
        return candicates[0]
    else:
        mhealth_root = get_mhealth_root(filepath)
        result = os.path.join(
            mhealth_root, 'DerivedCrossParticipants', 'offset_mapping.csv')
        if os.path.exists(result):
            return result
        else:
            return False


def find_pid_exceptions(filepath):
    mhealth_root = get_mhealth_root(filepath)
    result = os.path.join(
        mhealth_root, 'DerivedCrossParticipants', 'pid_exceptions.csv')
    if os.path.exists(result):
        return result
    else:
        return False


def find_orientation_corrections(filepath):
    pid_root = get_pid_root(filepath)
    candicates = glob(os.path.join(
        pid_root, '**', 'orientation_corrections.csv'), recursive=True)
    if len(candicates) == 1:
        return candicates[0]
    else:
        mhealth_root = get_mhealth_root(filepath)
        result = os.path.join(
            mhealth_root, 'DerivedCrossParticipants', 'orientation_corrections.csv')
        if os.path.exists(result):
            return result
        else:
            return False


def is_mhealth_filename(filepath):
    filename = os.path.basename(filepath)

    sensor_filename_pattern = '^' + CAMELCASE_PATTERN + '\-' + \
        CAMELCASE_PATTERN + \
        '\-' + VERSIONCODE_PATTERN + '\.' + \
        SID_PATTERN + '\-' + CAMELCASE_PATTERN + '\.' + \
        FILE_TIMESTAMP_PATTERN + '\.sensor\.csv$'

    annotation_filename_pattern = '^' + CAMELCASE_PATTERN + '\.' + \
        ANNOTATOR_PATTERN + '\-' + CAMELCASE_PATTERN + '\.' + \
        FILE_TIMESTAMP_PATTERN + '\.annotation\.csv$'

    sensor_matched = re.search(
        sensor_filename_pattern,
        filename
    )
    annotation_matched = re.search(
        annotation_filename_pattern,
        filename
    )
    return sensor_matched is not None or annotation_matched is not None


def get_pid(filepath):
    if is_mhealth_filepath(filepath):
        matched = re.search(MHEALTH_FILEPATH_PATTERN, filepath)
    elif is_mhealth_flat_filepath(filepath):
        matched = re.search(MHEALTH_FLAT_FILEPATH_PATTERN, filepath)
    else:
        return None
    return matched.group(1) if matched is not None else None


def get_sensor_type(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    result = filename.split('.')[0].split('-')[0]
    return result


def get_data_type(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    tokens = filename.split('.')[0]
    tokens = tokens.split('-')
    if len(tokens) >= 2:
        return tokens[1]
    else:
        return None


def get_version_code(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    tokens = filename.split('.')[0]
    tokens = tokens.split('-')
    if len(tokens) >= 3:
        return tokens[2]
    else:
        return None


def get_sid(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    return filename.split('.')[1].split('-')[0]


def get_file_type(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    return filename.split('.')[-2]


def get_file_timestamp(filepath):
    assert is_mhealth_filename(filepath)
    filename = os.path.basename(filepath)
    timestamp_str = filename.split('.')[-3]
    timestamp_str = timestamp_str[:-6]
    return datetime.datetime.strptime(timestamp_str, '%Y-%m-%d-%H-%M-%S-%f')


def get_session_start_time(filepath, filepaths):
    pid = get_pid(filepath)
    smallest = datetime.datetime.now()
    for path in filepaths:
        if get_pid(path) == pid:
            timestamp = get_file_timestamp(path)
            if timestamp < smallest:
                smallest = timestamp
    smallest = smallest.replace(microsecond=0, second=0, minute=0)
    return smallest


def get_session_end_time(filepath, filepaths):
    pid = get_pid(filepath)
    largest = datetime.datetime.fromtimestamp(100000)
    for path in filepaths:
        if get_pid(path) == pid:
            timestamp = get_file_timestamp(path)
            if timestamp > largest:
                largest = timestamp
    largest = largest.replace(microsecond=0, second=0,
                              minute=0) + datetime.timedelta(hours=1)
    return largest


def get_timezone(filepath):
    dt = get_file_timestamp(filepath)
    return dt.tzinfo


def get_timezone_name(filepath):
    dt = get_file_timestamp(filepath)
    return dt.strftime('%Z')


def get_init_placement(filepath, mapping_file):
    assert is_mhealth_filename(filepath)
    mapping = pd.read_csv(mapping_file)
    sid = get_sid(filepath)
    pid = get_pid(filepath)
    loc = mapping.loc[(mapping['PID'] == pid) & (
        mapping['SENSOR_ID'] == sid), 'LOCATION'].values[0]
    return loc


def auto_init_placement(filepath):
    assert is_mhealth_filename(filepath)
    mapping_file = find_location_mapping(filepath)
    if mapping_file:
        mapping = pd.read_csv(mapping_file)
    else:
        return None
    sid = get_sid(filepath)
    pid = get_pid(filepath)
    if 'PID' in mapping:
        mask = (mapping['PID'] == pid) & (mapping['SENSOR_ID'] == sid)
    else:
        mask = mapping['SENSOR_ID'] == sid
    loc = mapping.loc[mask, 'LOCATION']
    if loc.empty:
        return None
    else:
        return loc.values[0]
    return loc



import gpxpy
import gpxpy.gpx
import pandas as pd
import numpy as np


def convert_gpx(gpx_data):
    gpx = gpxpy.parse(gpx_data)
    rows = []
    for track in gpx.tracks:
        for segment in track.segments:
            for point in segment.points:
                ts_str = point.time.strftime('%Y-%m-%d %H:%M:%S.%f')
                ts_str = ts_str[:-3]
                row = pd.DataFrame(data={
                    "HEADER_TIME_STAMP": [ts_str],
                    "START_TIME": [ts_str],
                    "LATITUDE": [point.latitude],
                    "LONGITUDE": [point.longitude],
                    "HEIGHT": [point.elevation],
                })
                rows.append(row)

    # concatenate as dataframe
    csv = pd.concat(rows)
    return csv


def convert_csv(gps_csv_data):
    output = pd.DataFrame()
    output['HEADER_TIME_STAMP'] = gps_csv_data['LOCAL DATE'] \
        .str.cat(gps_csv_data['LOCAL TIME'], sep=' ') \
        .str.cat(gps_csv_data['MS'].map(
            lambda x: str(x).strip()), sep='.') \
        .astype('datetime64[ms]') \
        .map(lambda x: x.strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]).values
    output['START_TIME'] = output['HEADER_TIME_STAMP']
    output['LATITUDE'] = gps_csv_data['LATITUDE']
    output['LONGITUDE'] = gps_csv_data['LONGITUDE']
    output['HEIGHT'] = gps_csv_data['HEIGHT']
    output['SPEED'] = gps_csv_data['SPEED']
    output['QUALITY'] = gps_csv_data['VALID']
    return output



import pandas as pd
import os


def convert_activpal3_csv(data):
    if isinstance(data, str) and os.path.isfile(data):
        data = pd.read_csv(data, header=None)
    output = data.copy(deep=True)
    output = output.iloc[:, 0:4]
    output.columns = ['HEADER_TIME_STAMP', 'X', 'Y', 'Z']
    output["HEADER_TIME_STAMP"] = pd.to_datetime(
        output["HEADER_TIME_STAMP"] * 60 * 60 * 24,
        origin=pd.Timestamp('1899-12-30'), unit='s').astype('datetime64[ms]')
    output.iloc[:, 1:4] = (output.iloc[:, 1:4] - 127) / 2**8 * 4
    return output


if __name__ == '__main__':
    import sys
    o = convert_activpal3_csv(sys.argv[1])
    print(o.head())



from setuptools import setup, find_packages

setup(
    name='padar_converter',
    version='0.2.11',
    packages=find_packages(),
    include_package_data=True,
    description='Extension of converters for sensor data used in padar package',
    long_description=open('README.md').read(),
    install_requires=[
        "gpxpy>=1.3.2",
        "pandas>=0.23.0",
        "chardet>=3.0.4"
    ],
)